/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "screen_connector_remote_surface_evas/image_internal.h"

#define SC_TOOLKIT_HANDLE_TAG "SC_TOOLKIT_HANDLE_TAG"

namespace screen_connector {

Image::Image(Evas_Object* raw, IEventListener* listener,
             const std::string& instId, int pid, void* tag)
    : EvasObject(raw, true), listener_(listener) {
  evas_object_image_alpha_set(GetRaw(), 1);

  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MOUSE_IN,
      MouseInCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MOUSE_OUT,
      MouseOutCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MOUSE_DOWN,
      MouseDownCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MOUSE_UP,
      MouseUpCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MULTI_DOWN,
      TouchMultiDownCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MULTI_UP,
      TouchMultiUpCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MOUSE_MOVE,
      MouseMoveCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MOUSE_WHEEL,
      MouseWheelCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_SHOW,
      ShowCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_HIDE,
      HideCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_RESIZE,
      ResizeCB, this);
  evas_object_event_callback_add(GetRaw(), EVAS_CALLBACK_MOVE,
      MoveCB, this);

  /* Set data to use in accessibility */
  std::string plugId = instId + ":";

  plugId += std::to_string(pid);
  evas_object_data_set(GetRaw(), "___PLUGID", strdup(plugId.c_str()));
  evas_object_data_set(GetRaw(), SC_TOOLKIT_HANDLE_TAG, tag);
}

Image::~Image() {
  char* plugId =
      static_cast<char*>(evas_object_data_del(GetRaw(), "___PLUGID"));

  if (plugId)
    free(plugId);

  evas_object_data_del(GetRaw(), SC_TOOLKIT_HANDLE_TAG);

  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MOUSE_IN,
      MouseInCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MOUSE_OUT,
      MouseOutCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MOUSE_DOWN,
      MouseDownCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MOUSE_UP,
      MouseUpCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MULTI_DOWN,
      TouchMultiDownCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MULTI_UP,
      TouchMultiUpCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MOUSE_MOVE,
      MouseMoveCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MOUSE_WHEEL,
      MouseWheelCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_SHOW,
      ShowCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_HIDE,
      HideCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_RESIZE,
      ResizeCB, this);
  evas_object_event_callback_del_full(GetRaw(), EVAS_CALLBACK_MOVE,
      MoveCB, this);
}

void Image::MouseInCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnMouseIn(*img, eventInfo);
}

void Image::MouseOutCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnMouseOut(*img, eventInfo);
}

void Image::MouseDownCB(void* data, Evas* e, Evas_Object* obj,
                        void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnMouseDown(*img, eventInfo);
}

void Image::MouseUpCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnMouseUp(*img, eventInfo);
}

void Image::TouchMultiDownCB(void* data, Evas* e, Evas_Object* obj,
                             void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnTouchMultiDown(*img, eventInfo);
}

void Image::TouchMultiUpCB(void* data, Evas* e, Evas_Object* obj,
                           void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnTouchMultiUp(*img, eventInfo);
}

void Image::MouseMoveCB(void* data, Evas* e, Evas_Object* obj,
                        void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnMouseMove(*img, eventInfo);
}

void Image::MouseWheelCB(void* data, Evas* e, Evas_Object* obj,
                         void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnMouseWheel(*img, eventInfo);
}

void Image::ShowCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnShow(*img, eventInfo);
}

void Image::HideCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnHide(*img, eventInfo);
}

void Image::ResizeCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnResize(*img, eventInfo);
}

void Image::MoveCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo) {
  Image* img = static_cast<Image*>(data);

  img->listener_->OnMove(*img, eventInfo);
}

void Image::Update(std::shared_ptr<WlBuffer> tbm) {
  int width;
  int height;
  tbm_surface_h tbmSurface;
  Evas_Native_Surface ns;

  prev_buf_ = std::move(tbm);
  /* get tbm surface from buffer */
  tbmSurface = (tbm_surface_h)wl_buffer_get_user_data(prev_buf_->GetRaw());
  width = tbm_surface_get_width(tbmSurface);
  height = tbm_surface_get_height(tbmSurface);

  memset(&ns, 0, sizeof(Evas_Native_Surface));
  ns.version = EVAS_NATIVE_SURFACE_VERSION;
  ns.type = EVAS_NATIVE_SURFACE_TBM;
  ns.data.tbm.buffer = tbmSurface;

  evas_object_resize(GetRaw(), width, height);
  evas_object_image_size_set(GetRaw(), width, height);
  evas_object_image_native_surface_set(GetRaw(), &ns);
  evas_object_image_pixels_dirty_set(GetRaw(), EINA_TRUE);
}

}  // namespace screen_connector
